# -*- coding: utf-8 -*-
from __future__ import print_function
import codecs, csv, datetime, json, sys, urllib
try:
    from urllib import urlopen
except ImportError:
    from urllib.request import urlopen
try:
    import xmlrpclib
except ImportError:
    import xmlrpc.client as xmlrpclib

def count_packages(package_names):
    package_list = []
    for name in package_names:
        url = 'https://pypi.python.org/pypi/%s/json' % name
        reader = codecs.getreader("utf-8")
        package_data =  json.load(reader(urlopen(url)))
        downloads =  package_data['info']['downloads']
        print(name, ':', downloads['last_month'], 'downloads last_month')
        all_time = 0
        package_releases = package_data['releases']
        num_releases = len(package_releases)
        first_upload = datetime.datetime.now().isoformat()
        for version, release in package_releases.items():
            for downloadable in release:
                all_time += downloadable['downloads']
                upload_time = downloadable['upload_time']
                first_upload = min(first_upload, upload_time)
        package_list.append((downloads['last_month'],
                            downloads['last_week'],
                            downloads['last_day'],
                            all_time,
                            num_releases,
                            first_upload,
                            name))
    package_list.sort(reverse=True)
    print('================================')
    return package_list

def save_csv(package_list, name):
    with open('%s.csv' % name, 'w') as csv_out_file:
        writer =  csv.writer(csv_out_file)
        writer.writerow(['last month', 'last week', 'last day', 'all',
                    'num_releases', 'first_upload', 'name'])
        for p in package_list:
            print(p[0], p[3]//p[4], p[6])
            writer.writerow(p)

def count_user(username, role=None):
    client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')
    user_packages = client.user_packages(username)
    if role:
         package_names = [n[1] for n in user_packages
                            if n[0] == role]
    else:
        package_names = [n[1] for n in user_packages]
    package_names = list(set(package_names))
    package_list = count_packages(package_names)
    return package_list

def count_search(**kwargs):
    client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')
    packages = client.search(kwargs)
    package_names = [p['name'] for p in packages]
    package_list = count_packages(package_names)
    return package_list



if __name__ == '__main__':
    kwargs = dict(x.split('=', 1) for x in sys.argv[1:])
    package_list = count_search(**kwargs)
    save_csv(package_list, list(kwargs.values())[0])




from setuptools import setup, find_packages
import sys, os
from setuptools.command.test import test as TestCommand

version = '0.0'

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


setup(name='pypicount',
      version=version,
      description="Count download pypi packages",
      long_description=open('README.rst').read(),
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
      ],
      keywords='pypi cheeseshop',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='http://github.com/cleder/pypicount',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      tests_require=['pytest'],
      cmdclass = {'test': PyTest},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
#    Copyright (C) 2013  Christian Ledermann
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
import csv
import unittest
from .counter import count_search, count_user, save_csv

class BasicTestCase(unittest.TestCase):

    def test_count_user(self):
        self.assertTrue(len(count_user('christian.ledermann')) > 0)
        self.assertTrue(len(count_user('christian.ledermann', 'Maintainer')) > 0)

    def test_count_search(self):
        self.assertTrue(len(count_search(name='collective.geo')) > 0)

    def test_save_csv(self):
        package_list= [(1547, 244, 40, 12857, 9, u'2012-06-27T11:26:39', 'fastkml')]
        save_csv(package_list, 'test')






def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BasicTestCase))
    return suite

if __name__ == '__main__':
    unittest.main()
